/*  smplayer, GUI front-end for mplayer.
    Copyright (C) 2007 Ricardo Villalba <rvm@escomposlinux.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "keys.h"
#include <qkeysequence.h>
#include <qevent.h>

// MKey
MKey::MKey() {
	_function = "";
	_key = 0;
	_modifier = 0;
}

MKey::MKey( QString function, int key, int modifier ) {
	_function = function;
	_key = key;
	_modifier = modifier;
}

MKey::~MKey() {
}

QString MKey::printName() {
	QKeySequence k( _modifier | _key );
	return k;
}

QKeySequence MKey::keySequence() {
	QKeySequence k( _modifier | _key );
	return k;
}

bool MKey::check( QKeyEvent *e ) {
	//qDebug("MKey::check: %d", e->key() );

	if ( (e->key()==0) && (_modifier==0) ) {
		return (e->key() == _key);
	} else {
		int m=0;
		switch (e->state()) {
			//case (Qt::ShiftButton) : m = Shift; break;
			case (Qt::ControlButton) : m = Ctrl; break;
			case (Qt::AltButton) : m = Alt; break;
			default : m=0;
		}
		return ( (m == _modifier) && (e->key() == _key) );
	}
}


// Keys

Keys * Keys::keys_object=NULL;

Keys * Keys::obj() {
	if (keys_object==NULL) {
		qDebug( "Keys::obj: creating static obj" );
		keys_object = new Keys();
	}
	return keys_object;
}

Keys::Keys() {
	kl.append( MKey( "OSD", Qt::Key_O ) );
	//kl.append( MKey( "OSD", Qt::Key_O, MKey::Shift) );

	kl.append( MKey( "PAUSE", Qt::Key_P ) );
	kl.append( MKey( "PAUSE", Qt::Key_Space ) );

	//kl.append( MKey( "PLAY_PAUSE", Qt::Key_G ) );
	kl.append( MKey( "PLAY_PAUSE", Qt::Key_MediaPlay ) );

	kl.append( MKey( "FRAME_STEP", Qt::Key_Period ) );
	kl.append( MKey( "STOP", Qt::Key_MediaStop ) );

	kl.append( MKey( "FORWARD1", Qt::Key_Right ) ); // + 10 secs
	kl.append( MKey( "FORWARD1", Qt::Key_MediaNext ) ); // + 10 secs
	kl.append( MKey( "FORWARD2", Qt::Key_Up ) ); // + 1 minute
	kl.append( MKey( "FORWARD3", Qt::Key_Prior ) ); // + 10 minutes

	kl.append( MKey( "REWIND1", Qt::Key_Left ) ); // - 10 secs
	kl.append( MKey( "REWIND1", Qt::Key_MediaPrev ) ); // - 10 secs
	kl.append( MKey( "REWIND2", Qt::Key_Down ) ); // - 1 minute
	kl.append( MKey( "REWIND3", Qt::Key_Next ) ); // - 10 minutes

	kl.append( MKey( "MUTE", Qt::Key_M ) );

	kl.append( MKey( "INC_VOLUME", Qt::Key_0 ) );
	kl.append( MKey( "INC_VOLUME", Qt::Key_Asterisk ) );
	kl.append( MKey( "DEC_VOLUME", Qt::Key_9 ) );
	kl.append( MKey( "DEC_VOLUME", Qt::Key_Slash ) );

	kl.append( MKey( "INC_AUDIO_DELAY", Qt::Key_Plus ) );
	kl.append( MKey( "DEC_AUDIO_DELAY", Qt::Key_Minus ) );

	kl.append( MKey( "INC_SUB_DELAY", Qt::Key_X ) );
	kl.append( MKey( "DEC_SUB_DELAY", Qt::Key_Z ) );

	kl.append( MKey( "DEC_CONTRAST", Qt::Key_1 ) );
	kl.append( MKey( "INC_CONTRAST", Qt::Key_2 ) );

	kl.append( MKey( "DEC_BRIGHTNESS", Qt::Key_3 ) );
	kl.append( MKey( "INC_BRIGHTNESS", Qt::Key_4 ) );

	kl.append( MKey( "DEC_HUE", Qt::Key_5 ) );
	kl.append( MKey( "INC_HUE", Qt::Key_6 ) );

	kl.append( MKey( "DEC_SATURATION", Qt::Key_7 ) );
	kl.append( MKey( "INC_SATURATION", Qt::Key_8 ) );

	kl.append( MKey( "DEC_GAMMA", Qt::Key_1, MKey::Alt ) );
	kl.append( MKey( "INC_GAMMA", Qt::Key_2, MKey::Alt ) );

	kl.append( MKey( "DEC_SUBPOS", Qt::Key_R ) );
	kl.append( MKey( "INC_SUBPOS", Qt::Key_T ) );

	kl.append( MKey( "DEC_SPEED", Qt::Key_BracketLeft ) );
	kl.append( MKey( "INC_SPEED", Qt::Key_BracketRight ) );

	kl.append( MKey( "HALVE_SPEED", Qt::Key_BraceLeft ) );
	kl.append( MKey( "DOUBLE_SPEED", Qt::Key_BraceRight ) );

	kl.append( MKey( "NORMAL_SPEED", Qt::Key_Backspace ) );

	kl.append( MKey( "SCREENSHOT", Qt::Key_S ) );

	//kl.append( MKey( "AUDIO_NEXT", Qt::Key_NumberSign ) );
	kl.append( MKey( "AUDIO_NEXT", Qt::Key_H ) );
	kl.append( MKey( "SUBTITLE_NEXT", Qt::Key_J ) );
	kl.append( MKey( "CHAPTER_NEXT", Qt::Key_At ) );
	kl.append( MKey( "CHAPTER_PREV", Qt::Key_Exclam ) );
	//kl.append( MKey( "CHAPTER_PREV", Qt::Key_Exclam, MKey::Shift ) );

	kl.append( MKey( "PLAYLIST_NEXT", Qt::Key_Greater ) );
	//kl.append( MKey( "PLAYLIST_NEXT", Qt::Key_Greater, MKey::Shift ) );
	kl.append( MKey( "PLAYLIST_PREV", Qt::Key_Less ) );

	kl.append( MKey( "FULLSCREEN", Qt::Key_F ) );
	kl.append( MKey( "EXIT_FULLSCREEN", Qt::Key_Escape ) );

	// Options in menus
	kl.append( MKey( "OPEN_FILE", Qt::Key_F, MKey::Ctrl ) );
	kl.append( MKey( "OPEN_URL", Qt::Key_U, MKey::Ctrl ) );
	kl.append( MKey( "EXIT", Qt::Key_X, MKey::Ctrl ) );

	kl.append( MKey( "MPLAYER_LOG", Qt::Key_M, MKey::Ctrl ) );
	kl.append( MKey( "SMPLAYER_LOG", Qt::Key_S, MKey::Ctrl ) );

	kl.append( MKey( "INFO", Qt::Key_I, MKey::Ctrl ) );
	kl.append( MKey( "PREFERENCES", Qt::Key_P, MKey::Ctrl ) );

	kl.append( MKey( "VIDEO_EQUALIZER", Qt::Key_E, MKey::Ctrl ) );
	kl.append( MKey( "PLAYLIST", Qt::Key_L, MKey::Ctrl ) );

	kl.append( MKey( "COMPACT_MODE", Qt::Key_C, MKey::Ctrl ) );
}

Keys::~Keys() {
}

void Keys::list() {
	qDebug("Keys::list");
	MKeyList::Iterator it;
	for ( it = kl.begin(); it != kl.end(); ++it ) {
		qDebug( "  key: '%s' = '%s'", (*it).printName().utf8().data(), 
               (*it).function().utf8().data() );
	}
}

QString Keys::function(QKeyEvent *e) {
	MKeyList::Iterator it;
	for ( it = kl.begin(); it != kl.end(); ++it ) {
		if ( (*it).check(e) ) return (*it).function();
	}
	return "";
}

QKeySequence Keys::find( QString function ) {
	MKeyList::Iterator it;
	for ( it = kl.begin(); it != kl.end(); ++it ) {
		if ( (*it).function()==function ) return (*it).keySequence();
	}
	return 0;
}

